# Use always integer when comparing numbers in Python

* Status: [proposed]
* Deciders: [Bruno Friedmann]
* Date: [YYYY-MM-DD when the decision was last updated] <!-- optional -->

Technical Story: [#1982]

## Context and Problem Statement

[Describe the context and problem statement, e.g., in free form using two to three sentences. You may want to articulate the problem in form of a question.]
While doing wal file timestamp compare for incremental backups with the PostgreSQL plugin in python,
the file timestamp coming from os.stat function was expressed in float format, while the
`last_backup_time` `ROP` record was a simple integer. As float are architecture and language dependent
they are source of free interpretation and might lead to strange result in terms of comparison.


## Decision Drivers

* comparing different type or using conversion function rounding int(float()) are prone to error.
* comparing float type is only valid with strict greater or lower than. equal can't be used safely
* integer can be large as much as we need, and are safe to compare even cross-architecture

## Considered Options

* Compare file, job timestamp with integer (nanoseconds)
* Compare file, job timestamp with float (microsecond)
* Don't change the code mix of int, float, datetime representation.

## Decision Outcome

Chosen option: "[option 1]", because [justification. e.g., only option, which meets k.o. criterion decision driver | which resolves force force | ... | comes out best (see below)].

### Positive Consequences

* using always integer will ensure we are able to get a trustfull comparison.
* we will backup always the file we have to backup and not more

### Negative Consequences

* file timestamp are not tz aware, it is always the local timestamp.
* datetime (in Python) is still not able to support large integer (nanoseconds) since years.

## Pros and Cons of the Options

### [option 1]

[example | description | pointer to more information | ...] <!-- optional -->

* Good, because integer are safe in comparison whatever architecture is used
* Good, because integer are large in all case
* Bad, because integer with nanoseconds will force (in python) to check if _ns function are present,
  and force to do a division to retrieve the microsecond granularity.
* Bad, because it will need some efforts to standardize the code

### [option 2]

[example | description | pointer to more information | ...] <!-- optional -->

* Good, because float are easily transformable as `datetime` / `time` type
* Good, because float are the native format of `os.stat()` function without checking if nanoseconds
  are supported.
* Bad, because their representation is architecture dependent
* Bad, because their representation can't be compared with equality (only greater or lower)

### [option 3]

[example | description | pointer to more information | ...] <!-- optional -->

* Good, because the software will behave exactly as it is actually
* Good, because it offer freedom in implementation
* Bad, because it create deviant behavior depending of the platform and language used


## Links <!-- optional -->

* [Link type] [Link to ADR] <!-- example: Refined by [BADR-0005](0005-example.md) -->
* ... <!-- numbers of links can vary -->
